Program
    Uses
        cobra2D,
        keyset,
        poxml,
        svg

// Adapted from http://www.blitzbasic.com/Community/posts.php?topic=54327

Const
    lenDiv = 7.0
    ropeDivs = 64
    
Var
    
    // Rope Nodes
    pX: Array [ropeDivs] of Real
    pY: Array [ropeDivs] of Real
    pH: Array [ropeDivs] of Real
    pV: Array [ropeDivs] of Real
    
    lastx: Real
	lasty: Real
	lastv: Real
	lasth: Real
    
    // Circle
    cX: Real = 100
    cY: Real = 150 
    cR: Real = 100 

Procedure VertInit()
Var
    i: Integer    
Begin
    For i = 0 to (ropeDivs-1)
		pX[i] = (lenDiv * i)
		pY[i] = 0
        pH[i] = 0
        pV[i] = 0
    Next
End
        
Procedure VertLet()
Var
    i: Integer  
Begin
    
    // Apply to SVG
    For i = 0 to (ropeDivs-1)
        SetPolyX(rope.points, i, pX[i]-3)
        SetPolyY(rope.points, i, pY[i]-3)
        SetPolyX(rope.points, (2*ropeDivs)-1-i, pX[i]+3)
        SetPolyY(rope.points, (2*ropeDivs)-1-i, pY[i]+3)
    Next
    
End

Procedure UpdateRope()
Var
    i: Integer
    xd, yd, xdiff, ydiff, dist: Real  
Begin
    For i = 0 to (ropeDivs-1)
    
        // Gravity
        pV[i] = pV[i] + 0.5
            
//        ball\h = ball\h + (lastx - ball\x) * .05    ; Calcuate current velocity by comparing current link to previous links positions
//        ball\v = ball\v + (lasty - ball\y) * .05
        
        // Average the velocities between the current link and the previous link
        pV[i] = (pV[i] + lastv) * 0.5
        pH[i] = (pH[i] + lasth) * 0.5
        
        // "Air resistance"
        pV[i] = pV[i] * 0.95
        pH[i] = pH[i] * 0.95
        
        // Collisions with circle
        xdiff = 300 - pX[i]
        ydiff = 300 - pY[i]
        dist = Sqrt((xdiff * xdiff) + (ydiff * ydiff))
        If dist < 100 then
            pH[i] = 0
            pV[i] = 0
        Endif
        
        // Collisions with border
        If pY[i] < 0 Then pY[i] = 0; pV[i] = Abs(pV[i])
		If pX[i] < 0 Then pX[i] = 0; pH[i] = Abs(pH[i])
        If pX[i] > 640 Then pX[i] = 640; pH[i] = -Abs(pH[i])
        
        // Move
        pX[i] = pX[i] + pH[i]
        pY[i] = pY[i] + pV[i]

		// Calculate the distance between current & previous links
		xdiff = lastx - pX[i]
		ydiff = lasty - pY[i]
		dist = Sqrt((xdiff * xdiff) + (ydiff * ydiff))
		
		// Calculate the vector normal of where the link should be
		If Abs(dist) > 0.0 Then
			xd = (lastx - pX[i]) / dist
			yd = (lasty - pY[i]) / dist
		Else
			xd = 0
			yd = 0
		EndIf
		
		// Set the distance between current link and previous link
		xd = xd * lenDiv
		yd = yd * lenDiv

		// Calculate new position
		pX[i] = (lastx - xd)
		pY[i] = (lasty - yd)

		// Save current links coords and velocities
		lastx = pX[i]
		lasty = pY[i]
		lasth = pH[i]
		lastv = pV[i]
    Next
End



Var
    ms_start, ms, fm, cfps: Integer
    fd: Real = 1.0
    
Procedure SyncFramer()
Begin
    fm = Millisecs - ms
    ms = Millisecs
    fd = (fm / 1000.0) * 60.0
    fd = SmoothFrameDelta(fd) 
End

Function CalcFps(m1: Integer, m2: Integer) : Integer
Begin

    If ((m2 - m1) > 0) then
        result = (1000.0/(m2 - m1))
    Else
        result = 999
    Endif

End

Function SmoothFrameDelta(fd: Real) : Real ; export
Var
    avg, fd1, fd2, fd3, fd4, fd5, fd6, fd7, fd8, fd9: Static Real
Begin

    avg = (fd1 + fd2 + fd3 + fd4 + fd5 + fd6 + fd7 + fd8 + fd9 + fd) / 10.0 
    
    fd1 = fd2
    fd2 = fd3
    fd3 = fd4
    fd4 = fd5
    fd5 = fd6
    fd6 = fd7
    fd7 = fd8
    fd8 = fd9
    fd9 = fd
    
    If fd > avg then fd = avg
    result = fd

End

Var
    i, j, k, m: Integer = 0
    mySvg: ^svgs
    rope: ^svgPolygons
    canvas: Element
    
Begin

    SetAppName("SVG Rope Demo")
    OpenScreen(640, 480, 32, FALSE, COB_SHOWBORDER)
    VSync(1)
    Randomize(Millisecs)
    
    svgQuality = 1
    
    mySvg = LoadSvg('rope.svg')
    rope = GetPolygonById(mySvg, 'rope')
    rope.totalPoints = ropeDivs * 2
    
    
    canvas = CreateSprite(640, 480)
    PenColor(ToRGBA(0,0,0), canvas)
    SetFont("verdana", 8, 0, canvas)
    
    VertInit()
    
    SyncFramer()
    While (KeyDown(VK_ESCAPE) = FALSE)
        SyncFramer()
    
        CLS(ToRGBA(255,255,255), canvas)
        
        m = Millisecs
        
        lastx = MouseX()
	    lasty = MouseY()
	    lastv = MouseDY(TRUE)
	    lasth = MouseDX(TRUE)
        
        UpdateRope()
        VertLet()
        
        Ellipse(300, 300, 100, 100, ToRGBA(220, 220, 255), TRUE, canvas)
        Ellipse(300, 300, 100, 100, ToRGBA(0, 0, 0), FALSE, canvas)
        
        rope.updated = TRUE
        RenderSvgs(canvas) 
        
        Text(10, 10, "VSync enabled. FPS: "+cfps, canvas)
        Text(10, 25, "Real-time SVG Tech Demo", canvas)
        Text(10, 40, "Rope demo", canvas)
        Text(10, 55, "2010-09-05", canvas)
        Text(10, 70, "tophatstuff.co.uk", canvas)
    
        Flip
        //Pause(1)
        cfps = CalcFps(ms, Millisecs)
    Wend

End